__script__      = 'PyReloader'
__version__     = (0, 4, 0)
__year__        = 2021
__author__      = 'Zoey Mae'
__url__         = 'htts://git.barkshark.xyz/izaliamae/reload'
__license__     = 'Non-Violent Public License v5'
__license_url__ = 'https://git.pixie.town/thufie/NPL/src/tag/5/NPL.txt'

import json
from pathlib import Path


class DotDict(dict):
	def __init__(self, value=None, **kwargs):
		super().__init__()
		self.__setattr__ = self.__setitem__

		if isinstance(value, (str, bytes)):
			self.from_json(value)

		elif isinstance(value, dict):
			self.update(value)

		elif value:
			raise TypeError(f'The value must be a JSON string, list, dict, or another DotDict object, not {value.__class__}')

		if kwargs:
			self.update(kwargs)


	def __getattr__(self, k):
		try:
			return super().__getitem__(k)

		except KeyError:
			raise AttributeError(f'{self.__class__.__name__} object has no attribute {k}') from None


	def __setitem__(self, k, v):
		if isinstance(v, dict):
			v = DotDict(v)

		super().__setitem__(k, v)


	def __delattr__(self, k):
		try:
			dict.__delitem__(self, k)
		except KeyError:
			raise AttributeError(f'{self.__class__.__name__} object has no attribute {k}') from None


	def copy(self):
		return DotDict(self)


	def update(self, data):
		for k,v in data.items():
			self.__setitem__(k, v)


	def to_json(self, indent=None, **kwargs):
		if 'cls' not in kwargs:
			kwargs['cls'] = JsonEncoder

		return json.dumps(dict(self), indent=indent, **kwargs)


	def from_json(self, string):
		data = json.loads(string)
		self.update(data)


	def load_json(self, path):
		with open(path) as fd:
			self.update(json.load(fd))


	def save_json(self, path, indent=None):
		with open(path, 'w') as fp:
			fp.write(self.to_json(indent=indent))


class JsonEncoder(json.JSONEncoder):
	def default(self, obj):
		if not any(map(isinstance, [obj], [str, int, float, dict])):
			return str(obj)

		return json.JSONEncoder.default(self, obj)
