import json, os, shlex, sys, logging as logger

from pathlib import Path

from . import DotDict


defaults = {
	'bin': None,
	'args': [],
	'env': {},
	'path': Path.cwd(),
	'watch_ext': ['py', 'env'],
	'ignore_dirs': ['build', 'config', 'data'],
	'ignore_files': ['reload.py', 'test.py'],
	'log_level': 'INFO'
}


def setup_config(configfile=None):
	configfile = Path(configfile) if configfile else Path.cwd().joinpath('reload.json')
	config = DotDict(**defaults)

	if not configfile.exists():
		print('Cannot find "reload.json" in working directory. Creating config file and exiting...')

		new_config = defaults.copy()
		new_config['path'] = str(config.path)

		try:
			config.save_json(configfile, indent=4)

		except PermissionError:
			print('Failed to write config file because of permission error')

		sys.exit()

	config.load_json(configfile)
	config.path = Path(config.path or os.getcwd())
	config.log_level = getattr(logger, config.log_level.upper())
	config.ignore_dirs = [s.strip('/') for s in config.ignore_dirs]
	config.bin = Path(config.get('bin')).expanduser()

	if not config.bin:
		print('You forgot to specify a command')
		sys.exit()

	if not config.path.exists():
		print('Watch path doesn\'t exist:', config.path)
		sys.exit()

	console.level = config.log_level

	return config


logging = logger.getLogger('reloader')
logging.setLevel(logger.DEBUG)
console = logger.StreamHandler()
console.name = 'Console Log'
console.level = logger.INFO
console.formatter = logger.Formatter('%(asctime)s WATCHER: %(message)s', '%H:%M:%S')
logging.addHandler(console)
